/*
 * Copyright (c) 2019 Cloudera, Inc. All rights reserved.
 *
 * This code is provided to you pursuant to your written agreement with Cloudera, which may be the terms of the
 * Affero General Public License version 3 (AGPLv3), or pursuant to a written agreement with a third party authorized
 * to distribute this code.  If you do not have a written agreement with Cloudera or with an authorized and
 * properly licensed third party, you do not have any rights to this code.
 *
 * If this code is provided to you under the terms of the AGPLv3:
 *   (A) CLOUDERA PROVIDES THIS CODE TO YOU WITHOUT WARRANTIES OF ANY KIND;
 *   (B) CLOUDERA DISCLAIMS ANY AND ALL EXPRESS AND IMPLIED WARRANTIES WITH RESPECT TO THIS CODE, INCLUDING BUT NOT
 *       LIMITED TO IMPLIED WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE;
 *   (C) CLOUDERA IS NOT LIABLE TO YOU, AND WILL NOT DEFEND, INDEMNIFY, OR HOLD YOU HARMLESS FOR ANY CLAIMS ARISING
 *       FROM OR RELATED TO THE CODE; AND
 *   (D) WITH RESPECT TO YOUR EXERCISE OF ANY RIGHTS GRANTED TO YOU FOR THE CODE, CLOUDERA IS NOT LIABLE FOR ANY
 *       DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, PUNITIVE OR CONSEQUENTIAL DAMAGES INCLUDING, BUT NOT LIMITED
 *       TO, DAMAGES RELATED TO LOST REVENUE, LOST PROFITS, LOSS OF INCOME, LOSS OF BUSINESS ADVANTAGE OR
 *       UNAVAILABILITY, OR LOSS OR CORRUPTION OF DATA.
 */

package com.linkedin.kafka.cruisecontrol.monitor.sampling;

import com.cloudera.kafka.metrics.client.RawMetricsClient;
import com.cloudera.kafka.metrics.client.cm.CMMetricsClient;
import com.linkedin.cruisecontrol.metricdef.MetricDef;
import com.linkedin.kafka.cruisecontrol.config.BrokerCapacityConfigResolver;
import com.linkedin.kafka.cruisecontrol.config.constants.MonitorConfig;
import com.linkedin.kafka.cruisecontrol.exception.SamplingException;
import com.linkedin.kafka.cruisecontrol.metricsreporter.metric.CruiseControlMetric;
import org.apache.kafka.common.Cluster;
import org.apache.kafka.common.TopicPartition;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;

import static com.linkedin.kafka.cruisecontrol.monitor.sampling.MetricFetcherManager.BROKER_CAPACITY_CONFIG_RESOLVER_OBJECT_CONFIG;

public class ClouderaMetricsSampler implements MetricSampler {

    private static final Logger LOG = LoggerFactory.getLogger(ClouderaMetricsSampler.class);
    private CruiseControlMetricsProcessor _metricsProcessor;
    private RawMetricsClient _rawMetricsClient;

    @Override
    public Samples getSamples(
        Cluster cluster,
        Set<TopicPartition> assignedPartitions,
        long startTimeMs,
        long endTimeMs,
        SamplingMode mode,
        MetricDef metricDef,
        long timeout
    ) {
        List<CruiseControlMetric> rawMetrics = _rawMetricsClient.getMetrics(startTimeMs, endTimeMs);
        LOG.debug("Retrieved {} raw metrics", rawMetrics.size());

        for (CruiseControlMetric metric : rawMetrics) {
            _metricsProcessor.addMetric(metric);
        }

        try {
            if (rawMetrics.size() > 0) {
                LOG.debug("{} metrics added to be processed for {} with mode {}", rawMetrics.size(), assignedPartitions, mode);
                return _metricsProcessor.process(cluster, assignedPartitions, mode);
            } else {
                LOG.debug("No raw metrics were added. Returning empty Samples.");
                return new Samples(Collections.emptySet(), Collections.emptySet());
            }
        } finally {
            _metricsProcessor.clear();
        }
    }

    @Override
    public Samples getSamples(MetricSamplerOptions metricSamplerOptions) throws SamplingException {
        return this.getSamples(metricSamplerOptions.cluster(), metricSamplerOptions.assignedPartitions(),
                metricSamplerOptions.startTimeMs(), metricSamplerOptions.endTimeMs(), metricSamplerOptions.mode(),
                metricSamplerOptions.metricDef(), metricSamplerOptions.timeoutMs());
    }

    @Override
    public void configure(Map<String, ?> configs) {
        BrokerCapacityConfigResolver capacityResolver = (BrokerCapacityConfigResolver) configs.get(BROKER_CAPACITY_CONFIG_RESOLVER_OBJECT_CONFIG);
        if (capacityResolver == null) {
            throw new IllegalArgumentException("Metrics reporter sampler configuration is missing broker capacity config resolver object.");
        }
        boolean allowCpuCapacityEstimation = (Boolean) configs.get(MonitorConfig.SAMPLING_ALLOW_CPU_CAPACITY_ESTIMATION_CONFIG);
        _metricsProcessor = createMetricsProcessor(capacityResolver, allowCpuCapacityEstimation);

        _rawMetricsClient = createRawMetricsClient(configs);
    }

    @Override
    public void close() throws Exception {
        if (_rawMetricsClient != null) {
            _rawMetricsClient.close();
        }
    }

    CruiseControlMetricsProcessor createMetricsProcessor(BrokerCapacityConfigResolver capacityResolver, boolean allowCpuCapacityEstimation) {
        return new CruiseControlMetricsProcessor(capacityResolver, allowCpuCapacityEstimation);
    }

    RawMetricsClient createRawMetricsClient(Map<String, ?> configs) {
        RawMetricsClient rawMetricsClient = new CMMetricsClient();
        rawMetricsClient.initialize(configs);

        return rawMetricsClient;
    }

}
